@@ -25,8 +25,10 @@
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
 
-import os
 import json
+import os
+
+import openshot
 
 from classes import info
 from classes.app import get_app
@@ -257,6 +259,32 @@ class File(QueryObject):
         # Convert path to relative (based on current working directory of Python)
         return os.path.relpath(file_path, info.CWD)
 
+    def profile(self):
+        """ Get the profile of the file """
+        # Load file Json into Profile object
+        file_profile = openshot.Profile()
+        file_profile.SetJson(json.dumps(self.data))
+
+        if file_profile.info.display_ratio.num == 1 and file_profile.info.display_ratio.den == 1:
+            # Some audio / image files have inaccurate DAR - calculate from size and pixel ratio
+            file_profile.info.display_ratio = openshot.Fraction(round(file_profile.info.width * file_profile.info.pixel_ratio.ToFloat()), file_profile.info.height)
+            file_profile.info.display_ratio.Reduce()
+
+        # Iterate through all possible profiles
+        for profile_folder in [info.USER_PROFILES_PATH, info.PROFILES_PATH]:
+            for file in reversed(sorted(os.listdir(profile_folder))):
+                profile_path = os.path.join(profile_folder, file)
+                if os.path.isdir(profile_path):
+                    continue
+                try:
+                    # Load Profile
+                    profile = openshot.Profile(profile_path)
+                    if profile == file_profile:
+                        return profile
+                except RuntimeError as e:
+                    pass
+        return file_profile
+
 
 class Marker(QueryObject):
     """ This class allows Markers to be queried, updated, and deleted from the project data. """
