@@ -1,45 +1,41 @@
-""" 
+"""
  @file
  @brief This file contains the credits treeview, used by the about window
  @author Noah Figg <eggmunkee@hotmail.com>
  @author Jonathan Thomas <jonathan@openshot.org>
- 
+
  @section LICENSE
- 
+
  Copyright (c) 2008-2018 OpenShot Studios, LLC
  (http://www.openshotstudios.com). This file is part of
  OpenShot Video Editor (http://www.openshot.org), an open-source project
  dedicated to delivering high quality video editing and animation solutions
  to the world.
- 
+
  OpenShot Video Editor is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
- 
+
  OpenShot Video Editor is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
 
-import os
-from urllib.parse import urlparse
-
-from PyQt5.QtCore import QSize, Qt, QPoint
-from PyQt5.QtWidgets import QListView, QTreeView, QMessageBox, QAbstractItemView, QMenu, QSizePolicy, QHeaderView
+import webbrowser
+from PyQt5.QtCore import Qt
+from PyQt5.QtWidgets import QListView, QTreeView, QAbstractItemView, QSizePolicy, QHeaderView, QApplication
+from PyQt5.QtGui import QCursor
+from functools import partial
 
 from classes.logger import log
 from classes.app import get_app
 from windows.models.credits_model import CreditsModel
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
+from .menu import StyledContextMenu
 
 
 class CreditsTreeView(QTreeView):
@@ -65,6 +61,38 @@ class CreditsTreeView(QTreeView):
         if "website" not in self.columns:
             self.setColumnHidden(4, True)
 
+    def contextMenuEvent(self, event):
+        log.info('contextMenuEvent')
+        _ = get_app()._tr
+
+        # Get data model and selection
+        model = self.credits_model.model
+        row = self.indexAt(event.pos()).row()
+        if row != -1:
+            email = model.item(row, 3).text()
+            website = model.item(row, 4).text()
+
+            menu = StyledContextMenu(parent=self)
+            if email:
+                copy_action = menu.addAction(_("Copy E-mail"))
+                copy_action.triggered.connect(partial(self.CopyEmailTriggered, email))
+            if website:
+                github_action = menu.addAction(_("View Website"))
+                github_action.triggered.connect(partial(self.ViewWebsite, website))
+            menu.popup(QCursor.pos())
+
+    def CopyEmailTriggered(self, email=""):
+        log.info("CopyEmailTriggered")
+        clipboard = QApplication.clipboard()
+        clipboard.setText(email)
+
+    def ViewWebsite(self, website=""):
+        log.info("ViewWebsite")
+        try:
+            webbrowser.open(website)
+        except:
+            log.warning('Failed to launch web browser to %s' % website)
+
     def __init__(self, credits, columns, *args):
         # Invoke parent init
         QListView.__init__(self, *args)
