@@ -27,7 +27,7 @@
  """
 
 from PyQt5.QtCore import QSize, Qt, QPoint, QRegExp
-from PyQt5.QtGui import QDrag, QCursor
+from PyQt5.QtGui import QDrag, QCursor, QPixmap, QPainter, QIcon
 from PyQt5.QtWidgets import QListView, QAbstractItemView
 
 from classes import info
@@ -47,6 +47,7 @@ class FilesListView(QListView):
 
         # Set context menu mode
         app = get_app()
+        _ = app._tr
         app.context_menu_object = "files"
 
         index = self.indexAt(event.pos())
@@ -72,15 +73,32 @@ class FilesListView(QListView):
             file = File.get(id=file_id)
             if file and file.data.get("path").endswith(".svg"):
                 menu.addAction(self.win.actionEditTitle)
-                menu.addAction(self.win.actionDuplicateTitle)
+                menu.addAction(self.win.actionDuplicate)
                 menu.addSeparator()
 
             menu.addAction(self.win.actionPreview_File)
             menu.addSeparator()
-            menu.addAction(self.win.actionSplitClip)
-            menu.addAction(self.win.actionExportClips)
+            menu.addAction(self.win.actionSplitFile)
+            menu.addAction(self.win.actionExportFiles)
             menu.addSeparator()
             menu.addAction(self.win.actionAdd_to_Timeline)
+
+            # Add Profile menu
+            profile_menu = StyledContextMenu(title=_("Choose Profile"), parent=self)
+            profile_icon = get_app().window.actionProfile.icon()
+            profile_missing_icon = QIcon(":/icons/Humanity/actions/16/list-add.svg")
+            profile_menu.setIcon(profile_icon)
+
+            # Get file's profile
+            file_profile = file.profile()
+            if file_profile.info.description:
+                action = profile_menu.addAction(profile_icon, file_profile.info.description)
+                action.triggered.connect(lambda: get_app().window.actionProfile_trigger(file_profile))
+            else:
+                action = profile_menu.addAction(profile_missing_icon, _(f"Create Profile") + f": {file_profile.ShortName()}")
+                action.triggered.connect(lambda: get_app().window.actionProfileEdit_trigger(file_profile, duplicate=True))
+            menu.addMenu(profile_menu)
+
             menu.addAction(self.win.actionFile_Properties)
             menu.addSeparator()
             menu.addAction(self.win.actionRemove_from_Project)
@@ -89,6 +107,16 @@ class FilesListView(QListView):
         # Show menu
         menu.popup(event.globalPos())
 
+    def mouseDoubleClickEvent(self, event):
+        super(FilesListView, self).mouseDoubleClickEvent(event)
+        # Preview File, File Properties, or Split File (depending on Shift/Ctrl)
+        if int(get_app().keyboardModifiers() & Qt.ShiftModifier) > 0:
+            get_app().window.actionSplitFile.trigger()
+        elif int(get_app().keyboardModifiers() & Qt.ControlModifier) > 0:
+            get_app().window.actionFile_Properties.trigger()
+        else:
+            get_app().window.actionPreview_File.trigger()
+
     def dragEnterEvent(self, event):
         # If dragging urls onto widget, accept
         if not event.mimeData().hasUrls():
@@ -103,24 +131,54 @@ class FilesListView(QListView):
         # Get first column indexes for all selected rows
         selected = self.selectionModel().selectedRows(0)
 
-        # Get image of current item
-        current = self.selectionModel().currentIndex()
-        if not current.isValid() and selected:
-            current = selected[0]
-
-        if not current.isValid():
+        # Check if there are any selected items
+        if not selected:
             log.warning("No draggable items found in model!")
             return False
 
-        # Get icon from column 0 on same row as current item
-        icon = current.sibling(current.row(), 0).data(Qt.DecorationRole)
+        # Get icons from up to 3 selected items
+        icons = []
+        for i in range(min(3, len(selected))):
+            current = selected[i]
+            icon = current.sibling(current.row(), 0).data(Qt.DecorationRole)
+            if icon:
+                icons.append(icon.pixmap(self.drag_item_size))
 
-        # Start drag operation
+        # If no icons were retrieved, abort the drag
+        if not icons:
+            log.warning("No valid icons found for dragging!")
+            return False
+
+        # Calculate the total width of the composite pixmap including gaps
+        gap = 1  # 1 pixel gap between icons
+        total_width = (self.drag_item_size.width() * len(icons)) + (gap * (len(icons) - 1))
+
+        # Create a composite pixmap to hold the icons in a row
+        composite_pixmap = QPixmap(total_width, self.drag_item_size.height())
+        composite_pixmap.fill(Qt.transparent)  # Start with a transparent background
+
+        # Use a QPainter to draw the icons in a row with 1 pixel gap between them
+        painter = QPainter(composite_pixmap)
+        for idx, icon_pixmap in enumerate(icons):
+            x_offset = idx * (self.drag_item_size.width() + gap)  # Position each icon with a gap
+            painter.drawPixmap(int(x_offset), 0, icon_pixmap)
+        painter.end()
+
+        # Start the drag operation
         drag = QDrag(self)
-        drag.setMimeData(self.model().mimeData(selected))
-        drag.setPixmap(icon.pixmap(self.drag_item_size))
-        drag.setHotSpot(self.drag_item_center)
-        drag.exec_()
+
+        # Combine all selected items into the mime data
+        mime_data = self.model().mimeData(selected)
+        drag.setMimeData(mime_data)
+
+        # Set the composite pixmap for the drag operation
+        drag.setPixmap(composite_pixmap)
+
+        # Set the hot spot to the center of the composite pixmap
+        drag.setHotSpot(composite_pixmap.rect().center())
+
+        # Execute the drag operation
+        drag.exec_(supportedActions)
 
     # Without defining this method, the 'copy' action doesn't show with cursor
     def dragMoveEvent(self, event):
