@@ -30,12 +30,14 @@
 import os
 
 from PyQt5.QtCore import QSize, Qt, QPoint
-from PyQt5.QtGui import QDrag, QCursor
-from PyQt5.QtWidgets import QTreeView, QAbstractItemView, QMenu, QSizePolicy, QHeaderView
+from PyQt5.QtGui import QDrag, QCursor, QPixmap, QPainter, QIcon
+from PyQt5.QtWidgets import QTreeView, QAbstractItemView, QSizePolicy, QHeaderView
 
+from classes import info
 from classes.app import get_app
 from classes.logger import log
 from classes.query import File
+from .menu import StyledContextMenu
 
 
 class FilesTreeView(QTreeView):
@@ -47,13 +49,14 @@ class FilesTreeView(QTreeView):
 
         # Set context menu mode
         app = get_app()
+        _ = app._tr
         app.context_menu_object = "files"
 
         event.accept()
         index = self.indexAt(event.pos())
 
         # Build menu
-        menu = QMenu(self)
+        menu = StyledContextMenu(parent=self)
 
         menu.addAction(self.win.actionImportFiles)
         menu.addAction(self.win.actionThumbnailView)
@@ -73,12 +76,32 @@ class FilesTreeView(QTreeView):
             file = File.get(id=file_id)
             if file and file.data.get("path").endswith(".svg"):
                 menu.addAction(self.win.actionEditTitle)
-                menu.addAction(self.win.actionDuplicateTitle)
+                menu.addAction(self.win.actionDuplicate)
                 menu.addSeparator()
 
             menu.addAction(self.win.actionPreview_File)
-            menu.addAction(self.win.actionSplitClip)
+            menu.addSeparator()
+            menu.addAction(self.win.actionSplitFile)
+            menu.addAction(self.win.actionExportFiles)
+            menu.addSeparator()
             menu.addAction(self.win.actionAdd_to_Timeline)
+
+            # Add Profile menu
+            profile_menu = StyledContextMenu(title=_("Choose Profile"), parent=self)
+            profile_icon = get_app().window.actionProfile.icon()
+            profile_missing_icon = QIcon(":/icons/Humanity/actions/16/list-add.svg")
+            profile_menu.setIcon(profile_icon)
+
+            # Get file's profile
+            file_profile = file.profile()
+            if file_profile.info.description:
+                action = profile_menu.addAction(profile_icon, file_profile.info.description)
+                action.triggered.connect(lambda: get_app().window.actionProfile_trigger(file_profile))
+            else:
+                action = profile_menu.addAction(profile_missing_icon, _(f"Create Profile") + f": {file_profile.ShortName()}")
+                action.triggered.connect(lambda: get_app().window.actionProfileEdit_trigger(file_profile))
+            menu.addMenu(profile_menu)
+
             menu.addAction(self.win.actionFile_Properties)
             menu.addSeparator()
             menu.addAction(self.win.actionRemove_from_Project)
@@ -87,6 +110,20 @@ class FilesTreeView(QTreeView):
         # Show menu
         menu.popup(event.globalPos())
 
+    def mouseDoubleClickEvent(self, event):
+        # Get the index of the item at the click position
+        index = self.indexAt(event.pos())
+        if index.column() == 0:
+            # If column 0 (thumbnail) is double-clicked, trigger the custom actions
+            if int(get_app().keyboardModifiers() & Qt.ShiftModifier) > 0:
+                get_app().window.actionSplitFile.trigger()
+            elif int(get_app().keyboardModifiers() & Qt.ControlModifier) > 0:
+                get_app().window.actionFile_Properties.trigger()
+            else:
+                get_app().window.actionPreview_File.trigger()
+        else:
+            super(FilesTreeView, self).mouseDoubleClickEvent(event)
+
     def dragEnterEvent(self, event):
         # If dragging urls onto widget, accept
         if event.mimeData().hasUrls():
@@ -99,24 +136,54 @@ class FilesTreeView(QTreeView):
         # Get first column indexes for all selected rows
         selected = self.selectionModel().selectedRows(0)
 
-        # Get image of current item
-        current = self.selectionModel().currentIndex()
-        if not current.isValid() and selected:
-            current = selected[0]
-
-        if not current.isValid():
+        # Check if there are any selected items
+        if not selected:
             log.warning("No draggable items found in model!")
             return False
 
-        # Get icon from column 0 on same row as current item
-        icon = current.sibling(current.row(), 0).data(Qt.DecorationRole)
+        # Get icons from up to 3 selected items
+        icons = []
+        for i in range(min(3, len(selected))):
+            current = selected[i]
+            icon = current.sibling(current.row(), 0).data(Qt.DecorationRole)
+            if icon:
+                icons.append(icon.pixmap(self.drag_item_size))
 
-        # Start drag operation
+        # If no icons were retrieved, abort the drag
+        if not icons:
+            log.warning("No valid icons found for dragging!")
+            return False
+
+        # Calculate the total width of the composite pixmap including gaps
+        gap = 1  # 1 pixel gap between icons
+        total_width = (self.drag_item_size.width() * len(icons)) + (gap * (len(icons) - 1))
+
+        # Create a composite pixmap to hold the icons in a row
+        composite_pixmap = QPixmap(total_width, self.drag_item_size.height())
+        composite_pixmap.fill(Qt.transparent)  # Start with a transparent background
+
+        # Use a QPainter to draw the icons in a row with 1 pixel gap between them
+        painter = QPainter(composite_pixmap)
+        for idx, icon_pixmap in enumerate(icons):
+            x_offset = idx * (self.drag_item_size.width() + gap)  # Position each icon with a gap
+            painter.drawPixmap(int(x_offset), 0, icon_pixmap)
+        painter.end()
+
+        # Start the drag operation
         drag = QDrag(self)
-        drag.setMimeData(self.model().mimeData(selected))
-        drag.setPixmap(icon.pixmap(self.drag_item_size))
-        drag.setHotSpot(self.drag_item_center)
-        drag.exec_()
+
+        # Combine all selected items into the mime data
+        mime_data = self.model().mimeData(selected)
+        drag.setMimeData(mime_data)
+
+        # Set the composite pixmap for the drag operation
+        drag.setPixmap(composite_pixmap)
+
+        # Set the hot spot to the center of the composite pixmap
+        drag.setHotSpot(composite_pixmap.rect().center())
+
+        # Execute the drag operation
+        drag.exec_(supportedActions)
 
     # Without defining this method, the 'copy' action doesn't show with cursor
     def dragMoveEvent(self, event):
@@ -157,7 +224,7 @@ class FilesTreeView(QTreeView):
 
     def resize_contents(self):
         # Get size of widget
-        thumbnail_width = 78
+        thumbnail_width = 80
         tags_width = 75
 
         # Resize thumbnail and tags column
@@ -210,13 +277,14 @@ class FilesTreeView(QTreeView):
         self.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.setSelectionModel(self.files_model.selection_model)
+        self.setSortingEnabled(True)
 
         self.setAcceptDrops(True)
         self.setDragEnabled(True)
         self.setDropIndicatorShown(True)
 
         # Setup header columns and layout
-        self.setIconSize(QSize(75, 62))
+        self.setIconSize(info.TREE_ICON_SIZE)
         self.setIndentation(0)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         self.setStyleSheet('QTreeView::item { padding-top: 2px; }')
@@ -227,4 +295,4 @@ class FilesTreeView(QTreeView):
         self.files_model.ModelRefreshed.connect(self.refresh_view)
 
         # setup filter events
-        # self.files_model.model.itemChanged.connect(self.value_updated)
+        self.files_model.model.itemChanged.connect(self.value_updated)
