@@ -29,14 +29,15 @@ import os
 import json
 import functools
 from operator import itemgetter
+import sip
 
-from PyQt5.QtCore import Qt, QRectF, QLocale, pyqtSignal, pyqtSlot, QRect
+from PyQt5.QtCore import Qt, QRectF, QLocale, pyqtSignal, pyqtSlot
 from PyQt5.QtGui import (
-    QCursor, QIcon, QColor, QBrush, QPen, QPalette, QPixmap,
+    QIcon, QColor, QBrush, QPen, QPalette, QPixmap,
     QPainter, QPainterPath, QLinearGradient, QFont, QFontInfo,
 )
 from PyQt5.QtWidgets import (
-    QTableView, QAbstractItemView, QMenu, QSizePolicy,
+    QTableView, QAbstractItemView, QSizePolicy,
     QHeaderView, QItemDelegate, QStyle, QLabel,
     QPushButton, QHBoxLayout, QFrame, QFontDialog
 )
@@ -48,6 +49,7 @@ from classes.query import Clip, Effect, Transition
 
 from windows.models.properties_model import PropertiesModel
 from windows.color_picker import ColorPicker
+from .menu import StyledContextMenu
 
 import openshot
 
@@ -105,6 +107,17 @@ class PropertyDelegate(QItemDelegate):
         else:
             value_percent = 0.0
 
+        # Get theme colors
+        if get_app().theme_manager:
+            theme = get_app().theme_manager.get_current_theme()
+            if not theme:
+                log.warning("No theme loaded yet. Skip rendering properties widget.")
+                return
+            foreground_color = theme.get_color(".property_value", "foreground-color")
+            background_color = theme.get_color(".property_value", "background-color")
+        else:
+            log.warning("No ThemeManager loaded yet. Skip rendering properties widget.")
+
         # set background color
         painter.setPen(QPen(Qt.NoPen))
         if property_type == "color":
@@ -116,9 +129,9 @@ class PropertyDelegate(QItemDelegate):
         else:
             # Normal Keyframe
             if option.state & QStyle.State_Selected:
-                painter.setBrush(QColor("#575757"))
+                painter.setBrush(background_color)
             else:
-                painter.setBrush(QColor("#3e3e3e"))
+                painter.setBrush(background_color)
 
         if readonly:
             # Set text color for read only fields
@@ -126,7 +139,7 @@ class PropertyDelegate(QItemDelegate):
         else:
             path = QPainterPath()
             path.addRoundedRect(QRectF(option.rect), 15, 15)
-            painter.fillPath(path, QColor("#3e3e3e"))
+            painter.fillPath(path, background_color)
             painter.drawPath(path)
 
             # Render mask rectangle
@@ -137,8 +150,8 @@ class PropertyDelegate(QItemDelegate):
 
             # gradient for value box
             gradient = QLinearGradient(option.rect.topLeft(), option.rect.topRight())
-            gradient.setColorAt(0, QColor("#828282"))
-            gradient.setColorAt(1, QColor("#828282"))
+            gradient.setColorAt(0, foreground_color)
+            gradient.setColorAt(1, foreground_color)
 
             # Render progress
             painter.setBrush(gradient)
@@ -191,6 +204,13 @@ class PropertiesTableView(QTableView):
             self.selected_label = model.item(row, 0)
             self.selected_item = model.item(row, 1)
 
+        # Verify label has not been deleted
+        if (self.selected_label and sip.isdeleted(self.selected_label)) or \
+                (self.selected_item and sip.isdeleted(self.selected_item)):
+            log.debug("Property has been deleted, skipping")
+            self.selected_label = None
+            self.selected_item = None
+
         # Is the user dragging on the value column
         if self.selected_label and self.selected_item and \
                 self.selected_label.data() and type(self.selected_label.data()) == tuple:
@@ -243,9 +263,18 @@ class PropertiesTableView(QTableView):
                     # Get effect object
                     c = Effect.get(id=item_id)
 
-                if c and property_key in c.data:
+                if c and c.data:
                     # Grab the original data for this item/property
-                    self.original_data = c.data
+                    if property_key in c.data:
+                        self.original_data = c.data
+                    else:
+                        # If the property is not found, search in the tracked objects (if any)
+                        objects_dict = c.data.get("objects", {})
+                        for obj_id, obj in objects_dict.items():
+                            if property_key in obj:
+                                # use parent effect data (if match found)
+                                self.original_data = c.data
+                                break
 
             # For numeric values, apply percentage within parameter's allowable range
             if property_type in ["float", "int"] and property_name != "Track":
@@ -306,7 +335,7 @@ class PropertiesTableView(QTableView):
 
         # Enable video caching again
         openshot.Settings.Instance().ENABLE_PLAYBACK_CACHING = True
-        log.debug('mouseReleaseEvent: Start caching frames on timeline')
+        log.debug('mouseReleaseEvent: apply_last_action to history')
 
         # Add final update to undo/redo history
         get_app().updates.apply_last_action_to_history(self.original_data)
@@ -385,13 +414,22 @@ class PropertiesTableView(QTableView):
             # Ignore blank selections
             return
 
+        caption_model_label = caption_model_row[0]
+        caption_model_value = caption_model_row[1]
+
+        # Verify label has not been deleted
+        if (caption_model_label and sip.isdeleted(caption_model_label)) or \
+                (caption_model_value and sip.isdeleted(caption_model_value)):
+            log.debug("Property has been deleted, skipping")
+            return
+
         # Get data model and selection
-        cur_property = caption_model_row[0].data()
+        cur_property = caption_model_label.data()
         property_type = cur_property[1]["type"]
 
         # Save caption text
         if property_type == "caption" and cur_property[1].get('memo') != new_caption_text:
-            self.clip_properties_model.value_updated(caption_model_row[1], value=new_caption_text)
+            self.clip_properties_model.value_updated(caption_model_value, value=new_caption_text)
 
     def select_item(self, item_id, item_type):
         """ Update the selected item in the properties window """
@@ -433,6 +471,12 @@ class PropertiesTableView(QTableView):
         self.selected_item = selected_value
         frame_number = self.clip_properties_model.frame_number
 
+        # Skip any read-only properties
+        cur_property = selected_label.data()
+        readonly = cur_property[1]["readonly"]
+        if readonly:
+            return
+
         # Get translation object
         _ = get_app()._tr
 
@@ -458,6 +502,8 @@ class PropertiesTableView(QTableView):
             if property_key == "parent_effect_id" and not self.choices:
                 # Instantiate this effect
                 effect = Effect.get(id=clip_id)
+                if not effect:
+                    return
 
                 # Loop through timeline's clips
                 clip_choices = []
@@ -499,7 +545,14 @@ class PropertiesTableView(QTableView):
                     self.choices.append({"name": _("Clips"), "value": clip_choices, "selected": False, "icon": None})
 
             # Handle selected object options (ObjectDetection effect)
-            if property_key == "selected_object_index" and not self.choices:
+            if property_key in ["selected_object_index", "class_filter"] and not self.choices:
+                if property_key == "class_filter":
+                    # Use only class_name (if it has not already been added to the choices)
+                    tracked_object_menu_name = _("Tracked Classes")
+                    self.choices.append({"name": _("Clear"), "value": "", "selected": False, "icon": None})
+                else:
+                    tracked_object_menu_name = _("Tracked Objects")
+
                 # Get all visible object's indexes
                 timeline_instance = get_app().window.timeline_sync.timeline
                 # Instantiate the effect
@@ -508,18 +561,30 @@ class PropertiesTableView(QTableView):
                 visible_objects = json.loads(effect.GetVisibleObjects(frame_number))
                 # Add visible objects as choices
                 object_index_choices = []
-                for object_index in visible_objects["visible_objects_index"]:
-                    object_index_choices.append({
-                                "name": str(object_index),
-                                "value": str(object_index),
-                                "selected": False,
-                                "icon": None
-                            })
+                for enum_index, object_index in enumerate(visible_objects["visible_objects_index"]):
+                    class_name = visible_objects["visible_class_names"][enum_index]
+                    object_name = f"{class_name}: {object_index}"
+                    object_value = f"{object_index}"
+                    skip_choice = False
+                    if property_key == "class_filter":
+                        # Use only class_name (if it has not already been added to the choices)
+                        tracked_object_menu_name = _("Tracked Classes")
+                        object_name = f"{class_name}"
+                        object_value = f"{class_name}"
+                        skip_choice = any(d.get('name') == class_name for d in object_index_choices)
+
+                    if not skip_choice:
+                        object_index_choices.append({
+                                    "name": object_name,
+                                    "value": object_value,
+                                    "selected": False,
+                                    "icon": None
+                                })
                 if object_index_choices:
-                    self.choices.append({"name": _("Detected Objects"), "value": object_index_choices, "selected": False, "icon": None})
+                    self.choices.append({"name": tracked_object_menu_name, "value": object_index_choices, "selected": False, "icon": None})
 
             # Handle clip attach options
-            if property_key in ["parentObjectId", "child_clip_id"] and not self.choices:
+            if property_key in ["parentObjectId"] and not self.choices:
                 # Add all Clips as choices - initialize with None
                 tracked_choices = []
                 clip_choices = []
@@ -557,25 +622,25 @@ class PropertiesTableView(QTableView):
                             if effect.get("has_tracked_object"):
                                 # Instantiate the effect
                                 effect_instance = timeline_instance.GetClipEffect(effect["id"])
-                                # Get the visible object's ids
-                                visible_objects_id = json.loads(effect_instance.GetVisibleObjects(frame_number))["visible_objects_id"]
-                                for object_id in visible_objects_id:
-                                    # Get the Tracked Object properties
-                                    object_properties = json.loads(timeline_instance.GetTrackedObjectValues(object_id, 0))
-                                    x1 = object_properties['x1']
-                                    y1 = object_properties['y1']
-                                    x2 = object_properties['x2']
-                                    y2 = object_properties['y2']
-                                    # Get the tracked object's icon from the clip's icon
-                                    tracked_object_icon = icon_pixmap.copy(QRect(x1*icon_size, y1*icon_size, (x2-x1)*icon_size, (y2-y1)*icon_size)).scaled(icon_size, icon_size)
-                                    tracked_objects.append({"name": str(object_id),
-                                                            "value": str(object_id),
-                                                            "selected": False,
-                                                            "icon": QIcon(tracked_object_icon)})
-                            tracked_choices.append({"name": clip.data["title"],
-                                                  "value": tracked_objects,
-                                                  "selected": False,
-                                                  "icon": clip_instance_icon})
+                                # Get the indexes and IDs of the visible objects
+                                visible_objects = json.loads(effect_instance.GetVisibleObjects(frame_number))
+                                # Add visible objects as choices
+                                for enum_index, object_index in enumerate(visible_objects["visible_objects_index"]):
+                                    if "visible_class_names" in visible_objects:
+                                        class_name = visible_objects["visible_class_names"][enum_index]
+                                    else:
+                                        class_name = "Tracked Region"
+                                    tracked_objects.append({
+                                        "name": f"{class_name}: {object_index}",
+                                        "value": str(object_index),
+                                        "selected": False,
+                                        "icon": None
+                                    })
+
+                                tracked_choices.append({"name": clip.data["title"],
+                                                      "value": tracked_objects,
+                                                      "selected": False,
+                                                      "icon": clip_instance_icon})
                 self.choices.append({"name": _("None"), "value": "None", "selected": False, "icon": None})
                 if property_key == "parentObjectId" and tracked_choices:
                     self.choices.append({"name": _("Tracked Objects"), "value": tracked_choices, "selected": False, "icon": None})
@@ -684,7 +749,7 @@ class PropertiesTableView(QTableView):
             ]
 
             # Add menu options for keyframes
-            menu = QMenu(self)
+            menu = StyledContextMenu(parent=self)
             if self.property_type == "color":
                 Color_Action = menu.addAction(_("Select a Color"))
                 Color_Action.triggered.connect(functools.partial(self.Color_Picker_Triggered, cur_property))
@@ -707,56 +772,52 @@ class PropertiesTableView(QTableView):
                 Insert_Action.triggered.connect(self.Insert_Action_Triggered)
                 Remove_Action = menu.addAction(_("Remove Keyframe"))
                 Remove_Action.triggered.connect(self.Remove_Action_Triggered)
-                menu.popup(event.globalPos())
+                menu.addSeparator()
 
-            # Menu for choices
+            # Format menu nesting
             log.debug(f"Context menu choices: {self.choices}")
-            if not self.choices:
-                return
-            for choice in self.choices:
-                if type(choice["value"]) != list:
-                    # Just add root choice item
-                    Choice_Action = menu.addAction(_(choice["name"]))
-                    Choice_Action.setData(choice["value"])
-                    Choice_Action.triggered.connect(self.Choice_Action_Triggered)
-                    continue
+            self.menu = self.build_menu(self.choices, menu)
 
-                # Add sub-choice items (for nested choice lists)
-                # Divide into smaller QMenus (since large lists cover the entire screen)
-                # For example: Transitions -> 1 -> sub items
-                SubMenu = None
-                if choice.get("icon") is not None:
-                    SubMenuRoot = menu.addMenu(choice["icon"], choice["name"])
+            # Show context menu (if any options present)
+            # There is always at least 1 QAction in an empty menu though
+            if len(self.menu.children()) > 1:
+                self.menu.popup(event.globalPos())
+
+    def build_menu(self, data, parent_menu=None):
+        """Build a Context Menu, included nested sub-menus, and divide lists if too large"""
+        if parent_menu is None:
+            parent_menu = StyledContextMenu(parent=self)
+
+        # Get translation object
+        _ = get_app()._tr
+
+        SubMenuSize = 25
+        for choice in data:
+            if isinstance(choice["value"], list) and choice["value"]:
+                log.info("Add submenu: " + choice["name"])
+                if choice.get("icon"):
+                    SubMenuRoot = parent_menu.addMenu(QIcon(choice["icon"]), choice["name"])
                 else:
-                    SubMenuRoot = menu.addMenu(choice["name"])
+                    SubMenuRoot = parent_menu.addMenu(choice["name"])
 
-                SubMenuSize = 25
-                SubMenuNumber = 0
+                # Check if the list needs to be divided into sub-menus
                 if len(choice["value"]) > SubMenuSize:
-                    SubMenu = SubMenuRoot.addMenu(str(SubMenuNumber))
+                    for i in range(0, len(choice["value"]), SubMenuSize):
+                        range_label = f"{i + 1}-{min(i + SubMenuSize, len(choice['value']))}"
+                        SubMenu = SubMenuRoot.addMenu(range_label)
+                        self.build_menu(choice["value"][i:i + SubMenuSize], SubMenu)
                 else:
-                    SubMenu = SubMenuRoot
-                for i, sub_choice in enumerate(choice["value"], 1):
-                    # Divide SubMenu if it's item is a list
-                    if type(sub_choice["value"]) == list:
-                        SubSubMenu = SubMenu.addMenu(sub_choice["icon"], sub_choice["name"])
-                        for sub_sub_choice in sub_choice["value"]:
-                            Choice_Action = SubSubMenu.addAction(
-                                sub_sub_choice["icon"], sub_sub_choice["name"])
-                            Choice_Action.setData(sub_sub_choice["value"])
-                            Choice_Action.triggered.connect(self.Choice_Action_Triggered)
-                    else:
-                        if i % SubMenuSize == 0:
-                            SubMenuNumber += 1
-                            SubMenu = SubMenuRoot.addMenu(str(SubMenuNumber))
-                        Choice_Action = SubMenu.addAction(
-                            sub_choice["icon"], _(sub_choice["name"]))
-                        Choice_Action.setData(sub_choice["value"])
-                        Choice_Action.triggered.connect(self.Choice_Action_Triggered)
+                    self.build_menu(choice["value"], SubMenuRoot)
+            else:
+                # Single choice, not a list, add directly to the menu
+                log.info(" - Add choice: " + choice["name"])
+                Choice_Action = parent_menu.addAction(_(choice["name"]))
+                if choice.get("icon"):
+                    Choice_Action.setIcon(QIcon(choice["icon"]))
+                Choice_Action.setData(choice["value"])
+                Choice_Action.triggered.connect(self.Choice_Action_Triggered)
 
-            # Show choice menuk
-            log.debug(f"Display context menu: {menu.children()}")
-            menu.popup(event.globalPos())
+        return parent_menu
 
     def Bezier_Action_Triggered(self, preset=[]):
         log.info("Bezier_Action_Triggered: %s" % str(preset))
@@ -804,6 +865,14 @@ class PropertiesTableView(QTableView):
 
     def Insert_Action_Triggered(self):
         log.info("Insert_Action_Triggered")
+
+        # Verify label has not been deleted
+        if (self.selected_label and sip.isdeleted(self.selected_label)) or \
+                (self.selected_item and sip.isdeleted(self.selected_item)):
+            log.debug("Property has been deleted, skipping")
+            self.selected_label = None
+            self.selected_item = None
+
         if self.selected_item:
             current_value = QLocale().system().toDouble(self.selected_item.text())[0]
             self.clip_properties_model.value_updated(self.selected_item, value=current_value)
@@ -850,6 +919,7 @@ class PropertiesTableView(QTableView):
         self.original_data = None
         self.lock_selection = False
         self.prev_row = None
+        self.menu = None
 
         # Context menu icons
         self.bezier_icon = QIcon(QPixmap(os.path.join(info.IMAGES_PATH, "keyframe-%s.png" % openshot.BEZIER)))
@@ -893,7 +963,7 @@ class SelectionLabel(QFrame):
 
     def getMenu(self):
         # Build menu for selection button
-        menu = QMenu(self)
+        menu = StyledContextMenu(parent=self)
 
         # Get translation object
         _ = get_app()._tr
